@@ -100,8 +100,10 @@ def detailToAuthor(ad):
   return author
 
 # DB Classes
-from elixir import * 
 import sqlalchemy as sql
+import elixir as elixir
+import migrate as migrate
+import database
 
 # Patch from http://elixir.ematia.de/trac/wiki/Recipes/GetByOrAddPattern
 def get_by_or_init(cls, if_new_set={}, **params):
@@ -115,50 +117,74 @@ def get_by_or_init(cls, if_new_set={}, **params):
     result.set(**if_new_set)
   return result
 
-Entity.get_by_or_init = classmethod(get_by_or_init)
+elixir.Entity.get_by_or_init = classmethod(get_by_or_init)
 
-class Post(Entity):
-  using_options (tablename='posts')
-  feed        = ManyToOne('Feed')
-  title       = Field(Text)
-  post_id     = Field(Text)
-  content     = Field(Text)
-  date        = Field(DateTime)
-  unread      = Field(Boolean, default=True)
-  important   = Field(Boolean, default=False)
-  author      = Field(Text)
-  link        = Field(Text)
-  deleted     = Field(Boolean, default=False)
+root_feed=None
+
+def initDB():
+  global root_feed
+  REQUIRED_SCHEMA=1
+  # FIXME: show what we are doing on the UI
+  if not os.path.exists(database.dbfile): # Just create it
+    os.system('urssus_upgrade_db')
+  else: # May need to upgrade
+    try:
+      curVer=migrate.versioning.api.db_version(database.dbUrl, database.repo)
+    except:
+      curVer=0
+    if curVer < REQUIRED_SCHEMA:
+      print "UPGRADING from %s to %s"%(curVer, REQUIRED_SCHEMA)
+      os.system('urssus_upgrade_db')
+    
+  elixir.metadata.bind = database.dbUrl
+  elixir.setup_all()
+  elixir.session.flush()
+  root_feed=Feed.get_by_or_init(parent=None)
+  elixir.session.flush()
+  return True
+
+class Post(elixir.Entity):
+  elixir.using_options (tablename='posts')
+  feed        = elixir.ManyToOne('Feed')
+  title       = elixir.Field(elixir.Text)
+  post_id     = elixir.Field(elixir.Text)
+  content     = elixir.Field(elixir.Text)
+  date        = elixir.Field(elixir.DateTime)
+  unread      = elixir.Field(elixir.Boolean, default=True)
+  important   = elixir.Field(elixir.Boolean, default=False)
+  author      = elixir.Field(elixir.Text)
+  link        = elixir.Field(elixir.Text)
+  deleted     = elixir.Field(elixir.Boolean, default=False)
 
   def __repr__(self):
     if '<' in self.title:
       return h2t(self.title).strip()
     return unicode(self.title)
 
-class Feed(Entity):
-  using_options (tablename='feeds')
-  htmlUrl        = Field(Text)
-  xmlUrl         = Field(Text)
-  title          = Field(Text)
-  text           = Field(Text, default='')
-  description    = Field(Text)
-  children       = OneToMany('Feed')
-  parent         = ManyToOne('Feed')
-  posts          = OneToMany('Post', order_by="-date")
-  lastUpdated    = Field(DateTime, default=datetime(1970,1,1))
-  loadFull       = Field(Boolean, default=False)
+class Feed(elixir.Entity):
+  elixir.using_options (tablename='feeds')
+  htmlUrl        = elixir.Field(elixir.Text)
+  xmlUrl         = elixir.Field(elixir.Text)
+  title          = elixir.Field(elixir.Text)
+  text           = elixir.Field(elixir.Text, default='')
+  description    = elixir.Field(elixir.Text)
+  children       = elixir.OneToMany('Feed', inverse='parent')
+  parent         = elixir.ManyToOne('Feed')
+  posts          = elixir.OneToMany('Post', order_by="-date", inverse='feed')
+  lastUpdated    = elixir.Field(elixir.DateTime, default=datetime(1970,1,1))
+  loadFull       = elixir.Field(elixir.Boolean, default=False)
   # meaning of archiveType:
   # 0 = use default, 1 = keepall, 2 = use limitCount
   # 3 = use limitDays, 4 = no archiving
-  archiveType    = Field(Integer, default=0) 
-  limitCount     = Field(Integer, default=1000)
-  limitDays      = Field(Integer, default=60)
+  archiveType    = elixir.Field(elixir.Integer, default=0) 
+  limitCount     = elixir.Field(elixir.Integer, default=1000)
+  limitDays      = elixir.Field(elixir.Integer, default=60)
 
-  notify         = Field(Boolean, default=False)
-  markRead       = Field(Boolean, default=False)
-  icon           = Field(Binary, deferred=True)
+  notify         = elixir.Field(elixir.Boolean, default=False)
+  markRead       = elixir.Field(elixir.Boolean, default=False)
+  icon           = elixir.Field(elixir.Binary, deferred=True)
   # updateInterval -1 means use the app default, any other value, it's in minutes
-  updateInterval = Field(Integer, default=-1)
+  updateInterval = elixir.Field(elixir.Integer, default=-1)
   curUnread      = -1
 
   def __repr__(self):
@@ -195,12 +221,12 @@ class Feed(Entity):
         if post.important: continue # Don't delete important stuff
         post.deleted=True
         
-    session.flush()
+    elixir.session.flush()
     if expunge:
       # Delete all posts with deleted==True 
       for post in Post.query().filter(Post.feed==self).filter(Post.deleted==True).all():
         post.delete()
-      session.flush()
+      elixir.session.flush()
       
     # Force recount
     self.curUnread=-1
@@ -378,7 +404,7 @@ class Feed(Entity):
       # FIXME: handle 404, 403 whatever errors
       self.icon=urlopen(urlparse.urljoin(self.htmlUrl,'/favicon.ico')).read()
       open('/tmp/icon.ico', 'w').write(self.icon)
-    session.flush()
+    elixir.session.flush()
 
     
   def update(self):
@@ -422,7 +448,7 @@ class Feed(Entity):
          
         # Rudimentary NON-html detection
         if not '<' in content:
-          content='<pre>%s</pre>'%escape(content)
+          content=escape(content).replace('\n\n','<p>')
          
         # Author if available, else None
         author=''
@@ -461,7 +487,7 @@ class Feed(Entity):
       except KeyError:
         debug( post )
     self.lastUpdated=datetime.now()
-    session.flush()
+    elixir.session.flush()
       
     # Queue a notification if needed
     if posts and self.notify:
@@ -529,19 +555,6 @@ class Feed(Entity):
         return posts[ind-1]
     return None
 
-root_feed=None
-
-def initDB():
-  global root_feed
-  # This is just temporary
-  metadata.bind = "sqlite:///%s/urssus.sqlite"%config.cfdir
-  # metadata.bind.echo = True
-  setup_all()
-  if not os.path.exists("urssus.sqlite"):
-    create_all()
-  root_feed=Feed.get_by_or_init(parent=None)
-  session.flush()
-
 # UI Classes
 from PyQt4 import QtGui, QtCore, QtWebKit
 from ui.Ui_main import Ui_MainWindow
@@ -692,7 +705,7 @@ class FeedProperties(QtGui.QDialog):
       feed.archiveType=4
 
 
-    session.flush()
+    elixir.session.flush()
     QtGui.QDialog.accept(self)
  
 class MainWindow(QtGui.QMainWindow):
@@ -777,6 +790,8 @@ class MainWindow(QtGui.QMainWindow):
   def notificationClicked(self):
     if self.notifiedFeed:
       self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[self.notifiedFeed.id]))
+      self.activateWindow()
+      self.raise_()
 
   def loadPreferences(self):
     
@@ -802,9 +817,12 @@ class MainWindow(QtGui.QMainWindow):
     curPost=self.getCurrentPost()
     if not curPost: return
     info ("Deleting post: %s", curPost)
-    curPost.delete()
-    session.flush()
-    self.open_feed(self.ui.feeds.currentIndex())
+    if QtGui.QMessageBox.question(None, "Delete Article - uRSSus", 
+        'Are you sure you want to delete "%s"'%curPost, 
+        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No ) == QtGui.QMessageBox.Yes:
+      curPost.delete()
+      elixir.session.flush()
+      self.open_feed(self.ui.feeds.currentIndex())
 
   def on_actionMark_as_Read_triggered(self, i=None):
     # FIXME: handle selections
@@ -815,7 +833,7 @@ class MainWindow(QtGui.QMainWindow):
       info ("Marking as read post: %s", curPost)
       curPost.unread=False
       curPost.feed.curUnread-=1 
-      session.flush()
+      elixir.session.flush()
       self.updatePostItem(curPost)
       self.updateFeedItem(curPost.feed)
 
@@ -828,7 +846,7 @@ class MainWindow(QtGui.QMainWindow):
       info ("Marking as unread post: %s", curPost)
       curPost.unread=True
       curPost.feed.curUnread+=1
-      session.flush()
+      elixir.session.flush()
       self.updatePostItem(curPost)
       self.updateFeedItem(curPost.feed)
 
@@ -847,7 +865,7 @@ class MainWindow(QtGui.QMainWindow):
     if not curPost: return
     info ("Marking as important post: %s", curPost)
     curPost.important=True
-    session.flush()
+    elixir.session.flush()
     self.updatePostItem(curPost)
 
   def on_actionRemove_Important_Mark_triggered(self, i=None):
@@ -857,7 +875,7 @@ class MainWindow(QtGui.QMainWindow):
     if not curPost: return
     info ("Marking as not important post: %s", curPost)
     curPost.important=False
-    session.flush()
+    elixir.session.flush()
     self.updatePostItem(curPost)
 
 
@@ -954,7 +972,7 @@ class MainWindow(QtGui.QMainWindow):
       # if curFeed is a folder, add as child
       else:
         newFeed.parent=curFeed
-      session.flush()
+      elixir.session.flush()
       self.initTree()
       self.ui.feeds.setCurrentIndex(self.ui.feeds.model().indexFromItem(self.feedItems[newFeed.id]))
       self.on_actionEdit_Feed_triggered(True)
@@ -977,7 +995,7 @@ class MainWindow(QtGui.QMainWindow):
       # if curFeed is a folder, add as child
       else:
         newFolder.parent=curFeed
-      session.flush()
+      elixir.session.flush()
       self.initTree()
       self.ui.feeds.setCurrentIndex(self.ui.feeds.model().indexFromItem(self.feedItems[newFolder.id]))
 
@@ -1028,6 +1046,7 @@ class MainWindow(QtGui.QMainWindow):
       
   def unFilterPosts(self):
     self.textFilter=''
+    self.filterWidget.ui.filter.setText('')
     info("Text filter removed")
     self.open_feed(self.ui.feeds.currentIndex())
 
@@ -1186,6 +1205,8 @@ class MainWindow(QtGui.QMainWindow):
     self.ui.feeds.setCurrentIndex(index)
     feed=item.feed
     self.currentFeed=feed
+    # Scroll the feeds view so this feed is visible
+    self.ui.feeds.scrollTo(self.ui.feeds.model().indexFromItem(self.feedItems[self.currentFeed.id]))
     self.postItems={}
     self.posts=[]
     self.currentPost=None
@@ -1331,7 +1352,7 @@ class MainWindow(QtGui.QMainWindow):
     if post.unread:
       post.unread=False
       post.feed.curUnread-=1
-      session.flush()
+      elixir.session.flush()
     self.updateFeedItem(post.feed, parents=True)
     self.updatePostItem(post)
     if post.feed.loadFull and post.link:
@@ -1392,7 +1413,7 @@ class MainWindow(QtGui.QMainWindow):
           post.unread=False
           post.feed.curUnread-=1
           self.updatePostItem(post)
-      session.flush()
+      elixir.session.flush()
       self.updateFeedItem(item.feed, parents=True)
 
   def on_actionDelete_Feed_triggered(self, i=None):
@@ -1423,7 +1444,7 @@ class MainWindow(QtGui.QMainWindow):
         del(self.feedItems[item.feed.id])
         item.feed.delete()
         self.ui.feeds.model().removeRow(index.row(), index.parent())
-        session.flush()
+        elixir.session.flush()
         
   def on_actionOpen_Homepage_triggered(self, i=None):
     if i==None: return
@@ -1633,7 +1654,10 @@ def exportOPML(fname):
   opml.outlines=root._children
   opml.output(open(fname, 'w'))
     
-def importOPML(fname, parent=root_feed):
+def importOPML(fname, parent=None):
+  global root_feed
+  if parent == None:
+    parent=root_feed
 
   def importSubTree(parent, node):
     if node.tag<>'outline':
@@ -1664,7 +1688,7 @@ def importOPML(fname, parent=root_feed):
   tree = ElementTree.parse(fname)
   for node in tree.find('//body').getchildren():
     importSubTree(parent, node)
-  session.flush()
+  elixir.session.flush()
 
 # The feed updater (runs out-of-process)
 def feedUpdater(full=False):
